/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_EpBMTrVt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xmcouqJv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8qPVw64g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pLFzEKEU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_gaNbt2GH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Tmk54Fv6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_UkFCbYPp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_cVzYMQEm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_er4pTyTM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_uFnQyQAI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VX1chM9D_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1Xtm4nr4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_OUohDajr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Ot4WeO6D_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_LRCQhG07_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_wg5XMTBg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_9X4kM9GM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LLMYniyQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_qp8UpjnL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_85X6Lo2L_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_9r3drq3Z_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_cT2ZPSTt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JBexr8Hs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Qvy4irVU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ejjDOuuv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_0EGBzxMr_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_HBXgX3HU;
  SignalLine sLine_HCa0sTG5;
  SignalLine sLine_GkJgCXpo;
  ControlBinop cBinop_EpBMTrVt;
  ControlBinop cBinop_xmcouqJv;
  ControlBinop cBinop_8qPVw64g;
  ControlBinop cBinop_pLFzEKEU;
  ControlBinop cBinop_UkFCbYPp;
  ControlBinop cBinop_er4pTyTM;
  ControlBinop cBinop_uFnQyQAI;
  ControlBinop cBinop_VX1chM9D;
  ControlVar cVar_1Xtm4nr4;
  ControlVar cVar_OUohDajr;
  SignalVarf sVarf_lc4UJlHp;
  SignalVarf sVarf_MSrrcmgJ;
  SignalVarf sVarf_NoNqIAky;
  SignalVarf sVarf_wonrDyfZ;
  ControlBinop cBinop_LLMYniyQ;
  ControlBinop cBinop_cT2ZPSTt;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
